ig.module(
	'game.entities.ball'
)
.requires(
	'impact.entity'
)
.defines(function(){

EntityBall = ig.Entity.extend({
	
	animSheet: new ig.AnimationSheet( 'media/img/game/balls.png', 60, 60 ),
	
	startingDelay : new ig.Timer(),
	lostDelay : new ig.Timer(),
	
	size: {x: 60, y: 60},
	
	collides: ig.Entity.COLLIDES.NEVER,
	
	init: function( x, y, settings ) {
	    this.parent(x, y, settings);
		clearMouse();
		ig.global.ball.store.vel.x = 0;
		ig.global.ball.store.vel.y = 0;		
		this.startingDelay.set(0);
		this.lostDelay.set(0);
		this.lostDelay.pause();
		this.maxVel.x = 1000;
		this.maxVel.y = 5000;		
		this.setScale(ig.global.ball.height/60, ig.global.ball.width/60);
		this.pos.x = ig.global.game.center;
		this.pos.y = Math.round(ig.global.court.y + ig.global.player.height/2);
	},
		
	start: function(){
		this.addAnim( 'default', 1, [0] );		
		if(ig.global.ball.store.vel.x == 0){ //When game starts
			this.vel.x = 0;
			this.vel.y = ig.global.court.height * ig.global.game.speed;	
			if(Math.round(Math.random())==1){ig.game.playSound('hit1Sound');}else{ig.game.playSound('hit2Sound');};
		}else{ //When game continues
			this.vel.x  = ig.global.ball.store.vel.x;
			this.vel.y = ig.global.ball.store.vel.y;		
		}
	},
	
	draw: function(){
		this.parent();
		
		/* Countdown */
		var countdown = 0;

		if (this.startingDelay.delta() < 1){
			countdown = 3;
		}else if(this.startingDelay.delta() < 2){
			countdown = 2;
		}else if(this.startingDelay.delta() < 3){
			countdown = 1;
		};
		
		if(countdown > 0){
			ctx = ig.system.context;
			delay = 0.40;
			t = this.startingDelay.delta()%1 - delay;
			if( this.startingDelay.delta()%1 < delay ){
				textFontSize = Math.round(ig.global.game.height/2) * (0.8 + 0.2 * (this.startingDelay.delta()%1 / delay));
				ctx.globalAlpha = 1;
			}else{
				textFontSize = Math.round(ig.global.game.height/2) * (1 - 0.7 * ig.auxiliar.outExpo(t/(1 - delay)));
				ctx.globalAlpha = (1 - ig.auxiliar.outExpo(t/(1 - delay)));
			}
			
			textX = Math.round(ig.global.game.center);
			textY = Math.round(ig.global.game.height/2);
			ctx.fillStyle = 'yellow';
			ctx.font= textFontSize + "px HauracherellNC";
			ctx.textBaseline = 'middle';     
			ctx.textAlign = 'center'; 
			ctx.fillText( countdown, textX, textY );
			ctx.globalAlpha=1;
		}
	
	},

    update: function() {
	
	/* Active game */
	if (this.startingDelay.delta() > 3 && ig.global.game.paused == 1 && !ig.game.getEntityByName('pauseScreen') ){
		ig.global.game.paused = 0;
		this.startingDelay.pause();
		this.start();
	}
		
	
	/* Rival hits the ball */
	if( this.pos.y <= ig.global.pitch.y + ig.game.getEntityByName('rival').size.y * 0.75 && this.vel.y <= 0){
		if(Math.round(Math.random())==1){ig.game.playSound('hit1Sound');}else{ig.game.playSound('hit2Sound');};
		this.vel.x = ( (ig.global.court.x  + Math.round(ig.auxiliar.distribution(Math.random()) * ig.global.court.width) - this.pos.x ) * ig.global.game.speed );
		this.vel.y = ig.global.court.height * ig.global.game.speed;
	}
	
	/* Player hits the ball */
	if( 
		this.pos.y >= ig.game.getEntityByName('player').pos.y + ig.global.player.height * 0.30 &&
		this.pos.y < ig.game.getEntityByName('player').pos.y + ig.global.player.height &&
		this.vel.y > 0 &&
		(		
			(
			this.pos.x >= ig.game.getEntityByName('player').pos.x &&
			this.pos.x <= ig.game.getEntityByName('player').pos.x + ig.global.player.width
			)
			||
			(
			this.pos.x + this.size.x  >= ig.game.getEntityByName('player').pos.x &&
			this.pos.x + this.size.x <= ig.game.getEntityByName('player').pos.x + ig.global.player.width	
			)			
		)
	){
		if(Math.round(Math.random())==1){ig.game.playSound('hit1Sound');}else{ig.game.playSound('hit2Sound');};
		this.vel.x = ( (ig.global.court.x  + Math.round( ig.auxiliar.distribution(Math.random()) * ig.global.court.width) - this.pos.x ) * ig.global.game.speed );
		this.vel.y = -ig.global.game.speed * ig.global.court.height;
		ig.global.game.hits++;
		if(ig.global.game.hits == ig.global.game.maxHits){ ig.game.winGame(); };
	}
	
	/* Ball height effect */
	var resizer = 1 / (1 + Math.abs(this.pos.y - (ig.global.court.y + ig.global.court.height/2))/(ig.global.court.y + ig.global.court.height/2));
	this.setScale(ig.global.ball.height/60 * resizer, ig.global.ball.width/60 * resizer);
	
	/* Ball lost -> System Stop */
	if(this.pos.y >= ig.global.game.height && this.vel.y > 0 && ig.global.game.paused == 0){
		this.vel.x = 0;
		this.vel.y = 0;		
		ig.game.playSound('bounceSound');
		this.lostDelay.unpause();
	}
	
	/* LostGame */
	if(this.lostDelay.delta() > 1.5){ig.game.lostGame();};		

	if(this.pos.x <= ig.global.game.xmin){
		this.vel.x = + Math.abs(this.vel.x)
	}	  

	if(this.pos.x >= (ig.global.game.xmax - this.size.x)){
		this.vel.x = - Math.abs(this.vel.x)
	}	

      this.parent();
   }	
});

});