ig.module(
	'game.main' 
)
.requires(
	'impact.game',
	'impact.font',
	'plugins.scale',
	'plugins.auxiliar',
	'plugins.irontec',
	'game.globals',   
	'game.levels',   	
	'game.entities.homeScreen',
	'game.entities.tutorialScreen',   
	'game.entities.levelScreen',
	'game.entities.levelsMenuScreen',	
	'game.entities.pauseScreen',
	'game.entities.winScreen',   
	'game.entities.lostScreen',    
	'game.entities.topBar',
	'game.entities.pitch',   
	'game.entities.court',   
	'game.entities.player',   
	'game.entities.rival',
	'game.entities.ball',
	'game.entities.energizers',
	'game.entities.disturbers',
	'game.entities.damageBalls'	
)
.defines(function(){

loader = ig.Loader.extend({

	draw: function() {
	
		/* Loading Screen */
		var ctx = ig.system.context;
		
		/* Background */
		ctx.fillStyle= "#fff";
		ctx.fillRect( ig.global.game.xmin, 0, ig.global.game.width, ig.global.game.height );	

		/* Text (To force font preload) */
		var textFontSize = Math.floor(0.04 * ig.global.game.height);
		var textX = ig.global.game.center;
		var textY = 0.20 * ig.global.game.height ;
		ctx.fillStyle= "#ffffff";
		ctx.font= textFontSize + "px HauracherellNC";
		ctx.textBaseline = 'top';			
		ctx.textAlign = 'center';		
		ctx.fillText( ".", textX, textY );		

		/* Loading Logo */
		var loadingLogo = new ig.Image( 'media/img/ui/home/logo.png' );
		ig.system.context.drawImage( loadingLogo.data, ig.global.loadingScreen.logo.x, ig.global.loadingScreen.logo.y , ig.global.loadingScreen.logo.width, ig.global.loadingScreen.logo.height);
	
		/* Bar */
		ig.system.context.fillStyle= "#02406b";
		ig.system.context.fillRect( ig.global.loadingScreen.bar.x, ig.global.loadingScreen.bar.y, this.status * ig.global.loadingScreen.bar.width, ig.global.loadingScreen.bar.height );			

		}
}),
	
MyGame = ig.Game.extend({

	winSound : new ig.Sound( 'media/sounds/win.mp3' ),
	damageBallSound : new ig.Sound( 'media/sounds/damageBall.mp3' ),
	energizerSound : new ig.Sound( 'media/sounds/energizer.mp3' ),
	hit1Sound : new ig.Sound( 'media/sounds/hit2.mp3' ),
	hit2Sound : new ig.Sound( 'media/sounds/hit1.mp3' ),
	bounceSound : new ig.Sound( 'media/sounds/bounce.mp3' ), 

    init: function() {
		//ig.ua.iOS = 0; //MODO DEBUGGING IPHONE
		ig.game.initRemoteStorage();
		ig.irontec.initGame();	
		/* Replaying game after win exit */
		if (window.config.lastGameDate && (Math.abs(Date.parse(new Date()) - window.config.lastGameDate)/1000) < 20){
			this.launchLevelsMenu();
		}else{
			this.launchHome();
		}	
	},	
	
	initRemoteStorage : function(){
		if (window.configLoaded === false) {
			var that = this;
			window.setTimeout(function() {
				that.initRemoteStorage();
			},400);
			return;
		}
		if(!window.config) {
			window.config = {};
		}
		if (typeof window.config.sound == 'undefined') {
			window.config.sound = 1;
		}
		if (typeof window.config.maxLevel == 'undefined') {
			window.config.maxLevel = 1;
		}

		ig.global.game.maxLevel = window.config.maxLevel; /* Loading maxLevel */
		ig.irontec.saveToRemoteStorage();
	},	
	
	playSound : function(sound){
		if(window.config.sound == 1){
			if(!ig.ua.mobile){ eval('this.' + sound).play(); };
			if(ig.ua.iOS){ eval('this.' + sound).play(); };
			if(ig.ua.winPhone || ig.ua.android){ig.irontec.playSound(eval('this.' + sound));};
		}
	},

	disableSound : function(){
		window.config.sound = 0;
		ig.irontec.saveToRemoteStorage();
	},
	
	enableSound : function(){
		window.config.sound = 1;
		ig.irontec.saveToRemoteStorage();
	},	
	
    launchGame: function() {
		if(ig.game.getEntityByName('homeScreen')){ ig.game.getEntityByName('homeScreen').kill(); };
		if(ig.game.getEntityByName('lostScreen')){ ig.game.getEntityByName('lostScreen').kill(); };
		if(ig.game.getEntityByName('winScreen')){ ig.game.getEntityByName('winScreen').kill(); };		
		if(ig.game.getEntityByName('levelsMenuScreen')){ ig.game.getEntityByName('levelsMenuScreen').kill(); };	
		if(ig.game.getEntityByName('levelScreen')){ ig.game.getEntityByName('levelScreen').kill(); };	
		
		ig.global.game.paused = 1;
		ig.global.game.hits = 0; 
		
		var level = ig.global.game.level;
		
		ig.global.game.speed = ig.global.level[level].speed;
		ig.global.game.life = ig.global.level[level].maxLife;
		ig.global.game.maxLife = ig.global.level[level].maxLife;	
		ig.global.game.maxHits = ig.global.level[level].maxHits;		
		ig.global.energizers.minLife = ig.global.level[level].energizers.minLife;
		ig.global.energizers.duration = ig.global.level[level].energizers.duration;
		ig.global.energizers.cadence = ig.global.level[level].energizers.cadence;
		ig.global.disturbers.fallDuration = 2;
		ig.global.disturbers.fadeOutDuration = 2;
		ig.global.disturbers.cadence = ig.global.level[level].disturbers.cadence;
		ig.global.damageBalls.cadence = ig.global.level[level].damageBalls.cadence;
		
		ig.game.spawnEntity(EntityTopBar, ig.global.game.xmin, 0, {zIndex: 1, name : 'topBar'});		
		ig.game.spawnEntity(EntityPitch, ig.global.game.xmin, ig.global.pitch.y, {zIndex: 1, name : 'pitch'});					
		ig.game.spawnEntity(EntityCourt, ig.global.game.xmin, ig.global.court.y, {zIndex: 2, name : 'court'});		
		ig.game.spawnEntity(EntityRival, ig.global.game.center - ig.global.rival.width * 0.8, 0, {zIndex: 3, name : 'rival'});
		ig.game.spawnEntity(EntityBall, ig.global.game.center - ig.global.ball.width/2, 22, {zIndex: 4, name : 'ball'});
		ig.game.spawnEntity(EntityEnergizers, ig.global.game.xmin, 0, {zIndex: 5, name : 'energizers'});
		if(ig.global.disturbers.cadence>0){
			ig.game.spawnEntity(EntityDisturbers, ig.global.game.xmin, 0, {zIndex: 6, name : 'disturbers'});		
		}		
		if(ig.global.damageBalls.cadence>0){
			ig.game.spawnEntity(EntityDamageBalls, ig.global.game.xmin, 0, {zIndex: 7, name : 'damageBalls'});				
		}
		ig.game.spawnEntity(EntityPlayer, ig.global.game.center - ig.global.player.width/2, 0, {zIndex: 8, name : 'player'});	  
	},
	
    pauseGame: function() {
		ig.global.game.paused = 1;
		ig.global.ball.store.vel.x = ig.game.getEntityByName('ball').vel.x;
		ig.global.ball.store.vel.y = ig.game.getEntityByName('ball').vel.y;		
		ig.game.getEntityByName('ball').vel.x = 0;
		ig.game.getEntityByName('ball').vel.y = 0;
		ig.game.getEntityByName('energizers').durationTimer.pause();
		ig.game.getEntityByName('energizers').cadenceTimer.pause();	
		if(ig.global.disturbers.cadence>0){		
		ig.game.getEntityByName('disturbers').durationTimer.pause();
		ig.game.getEntityByName('disturbers').cadenceTimer.pause();		
		}
		ig.game.spawnEntity(EntityPauseScreen, ig.global.game.xmin, 0, {zIndex: 15, name : 'pauseScreen'});
	},		

    continueGame: function() {
		ig.game.getEntityByName('pauseScreen').kill();
		ig.game.getEntityByName('energizers').durationTimer.unpause();
		ig.game.getEntityByName('energizers').cadenceTimer.unpause();	
		if(ig.global.disturbers.cadence>0){		
		ig.game.getEntityByName('disturbers').durationTimer.unpause();
		ig.game.getEntityByName('disturbers').cadenceTimer.unpause();
		}
		ig.game.getEntityByName('ball').startingDelay.set(0);	
		ig.game.getEntityByName('ball').startingDelay.unpause();
	},		

    winGame: function() {
		ig.global.game.paused = 1;
		this.killGame();
		ig.game.spawnEntity(EntityWinScreen, ig.global.game.xmin, 0, {zIndex: 8, name : 'winScreen'});
	},	

    lostGame: function() {
		ig.global.game.paused = 1;
		this.killGame();
		ig.game.spawnEntity(EntityLostScreen, ig.global.game.xmin, 0, {zIndex: 8, name : 'lostScreen'});
	},
	
    exitPausedGame: function() {
		ig.irontec.cancelGame();		
		this.launchHome();
	},	
	
	killGame : function(){
		if(ig.game.getEntityByName('disturbers')){	
			ig.game.getEntityByName('disturbers').durationTimer.pause();
			ig.game.getEntityByName('disturbers').cadenceTimer.pause();	
			ig.game.getEntityByName('disturbers').kill();			
		}
		if(ig.game.getEntityByName('energizers')){	
			ig.game.getEntityByName('energizers').durationTimer.pause();
			ig.game.getEntityByName('energizers').cadenceTimer.pause();	
			ig.game.getEntityByName('energizers').kill();			
		}		
		if(ig.game.getEntityByName('damageBalls')){	
			ig.game.getEntityByName('damageBalls').kill();			
		}			
		if(ig.game.getEntityByName('player')){
			ig.game.getEntityByName('topBar').kill();
			ig.game.getEntityByName('pitch').kill();
			ig.game.getEntityByName('court').kill();
			ig.game.getEntityByName('rival').kill();
			ig.game.getEntityByName('ball').kill();
			ig.game.getEntityByName('player').kill();
		};		
	},
	
	exit : function(){
		if(ig.game.getEntityByName('winScreen')){ ig.game.getEntityByName('winScreen').kill(); };	
		if(ig.game.getEntityByName('lostScreen')){ ig.game.getEntityByName('lostScreen').kill(); };
		ig.irontec.exitGame();
	},
	
    launchHome: function() {
		this.killGame();
		if(ig.game.getEntityByName('lostScreen')){ ig.game.getEntityByName('lostScreen').kill(); };		
		if(ig.game.getEntityByName('winScreen')){ ig.game.getEntityByName('winScreen').kill(); };				
		if(ig.game.getEntityByName('pauseScreen')){ ig.game.getEntityByName('pauseScreen').kill(); };
		if(ig.game.getEntityByName('tutorialScreen')){ ig.game.getEntityByName('tutorialScreen').kill(); };		
		if(ig.game.getEntityByName('levelsMenuScreen')){ ig.game.getEntityByName('levelsMenuScreen').kill(); };				
		ig.game.spawnEntity(EntityHomeScreen, ig.global.game.xmin, 0, {zIndex: 10, name : 'homeScreen'});
	},
	
    launchTutorial: function() {
		ig.game.getEntityByName('homeScreen').kill();
		ig.game.spawnEntity(EntityTutorialScreen, ig.global.game.xmin, 0, {zIndex: 15, name : 'tutorialScreen'});	
	},		
 
    launchLevelsMenu: function() {
		if(ig.game.getEntityByName('winScreen')){ ig.game.getEntityByName('winScreen').kill(); };
		if(ig.game.getEntityByName('homeScreen')){ ig.game.getEntityByName('homeScreen').kill(); };
		if(ig.game.getEntityByName('levelScreen')){ ig.game.getEntityByName('levelScreen').kill(); };		
		ig.game.spawnEntity(EntityLevelsMenuScreen, ig.global.game.xmin, 0, {zIndex: 15, name : 'levelsMenuScreen'});	
	},	
 
    launchLevel: function() {
		if(ig.game.getEntityByName('levelsMenuScreen')){ ig.game.getEntityByName('levelsMenuScreen').kill(); };
		ig.game.spawnEntity(EntityLevelScreen, ig.global.game.xmin, 0, {zIndex: 15, name : 'levelScreen'});	
	},
	
	saveLastGameDate: function(){
		window.config.lastGameDate = new Date();
		ig.irontec.saveToRemoteStorage();
	},
	
	increaseMaxLevel : function(){
		if(ig.global.game.maxLevel<12 && ig.global.game.level + 1 > ig.global.game.maxLevel ){
			ig.global.game.maxLevel = ig.global.game.level + 1;
			window.config.maxLevel = ig.global.game.maxLevel;
			ig.irontec.saveToRemoteStorage();
		};
	}	
		  
});

ig.Sound.use = [ig.Sound.FORMAT.MP3];
ig.main( '#canvas', MyGame, 60,  window.innerWidth, window.innerHeight, 1, loader);

});
