ig.module(
	'game.entities.bikers'
)
.requires(
	'impact.entity'
)
.defines(function(){

EntityBikers = ig.Entity.extend({
	
	collides: ig.Entity.COLLIDES.NEVER,	
	
	sprintTimer : new ig.Timer,
	
	biker : [], type : [], order : [], scores : [], ranking : [],
	
	init: function( x, y, settings ) {
		this.parent(x, y, settings);
		
		clearMouse();	

		this.type = [0,0,0,1,1,1,2,2,2,3,4,4,4,5,6];
		for(var j, x, i = this.type.length; i; j = Math.floor(Math.random() * i), x = this.type[--i], this.type[i] = this.type[j], this.type[j] = x);

		for(var i=0; i<this.type.length; i++){
			var yPos = Math.round((0.15 + (0.55/this.type.length) * i) * ig.global.game.height);
			var height = Math.round(0.25 * ig.global.game.height);
			var xPos = Math.round(Math.random() * 0.1 * ig.global.game.width);
			if(this.type[i] == 3 || this.type[i] == 6){ xPos = - 0.3 * ig.global.game.width };
			if(this.type[i] == 5){ xPos = ig.global.game.width };
			if(this.type[i] <=4){
				this.biker[i] = new ig.BikerAnimation(i,this.type[i], 0, 6, 0.1, xPos, yPos, height);
			}else{
				this.biker[i] = new ig.ObstacleAnimation(i,this.type[i], 0, 6, 0.1, xPos, yPos, height);
			};
		};
		
		this.reorder();
	},
	
	reorder : function(){
		
		/* this.order arrays helps to order all bikers in z axis */
		for(var i=0;i<this.biker.length;i++){
			this.order[i]={biker : i, y : this.biker[i].y + this.biker[i].h[this.biker[i].frame]};
		}	
		this.order.sort(function(a,b) {
			if (a.y < b.y)
				return -1;
			if (a.y > b.y)
				return 1;
			return 0;
		});
		
	},
	
	HitTest : function(x,y){ 
		/* Each biker area has being reduced to some circle areas */
		/* This function calculates if click position is inside is at least one of these circles */
		/* We use distance equation in 2d euclidean space:  d(p1,p2) = root((p1.x-p2.x)^2 + (p1.y-p2.y)^2). */
		var spidioArea = [[178,58,50],[144,45,44],[134,104,66],[176,144,42],[55,144,42]];
		var bikersArea = [[158,38,38],[182,87,15],[118,105,56],[176,151,41],[55,141,41]];
		var boyArea = [[62,118,65],[82,89,45],[122,100,56],[163,92,27],[115,154,70],[68,174,32],[160,174,32]];
		var motorbikeArea = [[94,16,29],[133,41,72],[117,115,107],[58,148,47],[170,154,47],[71,168,37],[181,168,39]];				
		var hit=0;
		
		for(var c=0;c<this.order.length;c++){
			var actualBiker = this.biker[this.order[c].biker];
			if(actualBiker.type < 3) { var hitArea = bikersArea };
			if(actualBiker.type == 3) { var hitArea = spidioArea };
			if(actualBiker.type == 4) { var hitArea = bikersArea };
			if(actualBiker.type == 5) { var hitArea = boyArea };
			if(actualBiker.type == 6) { var hitArea = motorbikeArea };
			for(var i=0; i<hitArea.length; i++){
				var centerX = actualBiker.x + hitArea[i][0] * actualBiker.w[actualBiker.frame]/actualBiker.sw[actualBiker.frame];
				var centerY = actualBiker.y + hitArea[i][1] * actualBiker.w[actualBiker.frame]/actualBiker.sw[actualBiker.frame];	
				var r = hitArea[i][2] * actualBiker.w[actualBiker.frame]/actualBiker.sw[actualBiker.frame];
				if(Math.sqrt(Math.pow((x-centerX),2) + Math.pow((y-centerY),2)) < r){ hit=actualBiker.id; };
			};
		}
		return hit;
	},	

	draw: function() {  


		for(var i=0;i<this.biker.length;i++){
			this.biker[this.order[i].biker].draw();
		}
		
		for(var i=0;i<=this.scores.length-1;i++){
			if(this.scores[i].alpha > 0){this.scores[i].draw();};
		}		
		
	},	
	
	update: function() {
	
		this.reorder();

		for(var i=0;i<this.biker.length;i++){
			this.biker[i].update();
		}
		
		// When all bikers has crossed finish line, game finishes
		if( 
			this.ranking.length > 9 && 
			ig.game.getEntityByName('background').finishLine1.minX[0] < -ig.global.game.width/2 && 
			ig.game.finished==0 
		){
			ig.game.finish();
		}		
		
		// Active random biker sprint if we aren't in final Sprint
		if(this.sprintTimer.delta()>ig.global.game.bikerSprintCadence && ig.game.timer.delta() < ig.global.game.finalSprint && ig.game.timer.delta()<0){
			var randomBiker = Math.round(Math.random() * this.biker.length);
			if( this.biker[randomBiker].sprint == 0 ){
				if( this.biker[randomBiker].type < 3 ){ /* Non movistar bikers */
					this.biker[randomBiker].sprint = 1;
					this.sprintTimer.set(0);
					var newSpeed = 3 * (Math.abs(this.biker[randomBiker].speed));
					if(newSpeed < 8){ newSpeed = 8 };
					this.biker[randomBiker].speed = newSpeed;
				}
				if( this.biker[randomBiker].type == 4){ /* Movistar biker */
					this.biker[randomBiker].sprint = 1;
					this.sprintTimer.set(0);
					this.biker[randomBiker].speed=0;
					TweenMax.to(this.biker[randomBiker], 1, {x : 0.25 * ig.global.game.width + 0.1 * Math.random(), ease:Linear.easeNone});	
					this.biker[randomBiker].hand = new ig.UIObject("Hand.png", ui, this.biker[randomBiker].x + 0.8 * this.biker[randomBiker].w[this.frame], this.biker[randomBiker].y + 0.7 * this.biker[randomBiker].h[this.biker[randomBiker].frame], 0.2 * this.biker[randomBiker].w[this.biker[randomBiker].frame],1);
					var newY = 1.04 * this.biker[randomBiker].hand.y;
					TweenMax.to(this.biker[randomBiker].hand, 0.8, {alpha : 0.5, y : newY, repeat:-1, yoyo:true, ease:Linear.easeNone});								}				
			}
		}		
	
		this.parent();
   }	
});

});