/*
* ImpactJS Biker Animation Plugin
* Alex Maestre - August 2014
*
*/
ig.module(
    'plugins.bikerAnimation'
)
.requires(
	'impact.image',
	'impact.timer'
)

.defines(function() { "use strict";
	
	ig.BikerAnimation = ig.Class.extend({
	
		atlasPNG : new ig.Image( 'media/img/bikers.png' ),		
		
		delayTimer : null, timer: null, finished : 0, sprint : 0, storedSpeed : 0, hand : null,
		type : null, minFrame : 1, maxFrame : 1, frame : 1, frameTime : 1, id: null, speed : null, sx : [], sy : [], sw : [], sh : [], w : [], h : [], x:[], y : [], rotated : [], 
		
		init : function(id, type, minFrame, maxFrame, frameTime, x, y, finalHeight){
			var filename = '';
			switch(type){
				case 0 : filename = 'rider0'; this.speed = 4 + Math.random() * 4; break;
				case 1 : filename = 'rider1'; this.speed =  4 + Math.random() * 4; break;
				case 2 : filename = 'rider2'; this.speed =  4 + Math.random() * 4; break;
				case 3 : filename = 'spidio'; this.speed = 0; break;
				case 4 : filename = 'movistar0'; this.speed =  4 + Math.random() * 4; break;
			}
			this.id = id;
			this.type = type;
			this.timer = new ig.Timer();
			this.frameTime = frameTime;
			this.minFrame = minFrame;
			this.maxFrame = maxFrame;
			this.x = x;
			this.y = y;			
			for(var f=minFrame; f<=maxFrame; f++){
				var frame = bikers[filename + f + ".png"]["frame"].replace(/{/g,'').replace(/}/g,'').split(",");
				this.sx[f] = frame[0];
				this.sy[f] = frame[1];
				this.sw[f] = frame[2];
				this.sh[f] = frame[3];
				this.w[f] = Math.round(finalHeight * frame[2]/frame[3]);
				this.h[f] = Math.round(finalHeight);		
			}
			if(this.type == 3){ //Spidio sprints' starting timer
				this.delayTimer = new ig.Timer();				
			}
			if(this.type != 3){ //Movistar and Other bikers bunch random movement delay timer
				this.delayTimer = new ig.Timer(); //
				this.delayTimer.set();
				this.delayTimer.pause();
			}		
		},
		
		draw : function (){

			var ctx = ig.system.context;
			ctx.drawImage(this.atlasPNG.data, this.sx[this.frame], this.sy[this.frame], this.sw[this.frame], this.sh[this.frame], this.x, this.y, this.w[this.frame], this.h[this.frame]);

			//Draw hand if it exists
			if(this.hand && this.type==4){
				this.hand.draw();
			}
		
		},
		
		update : function(){
			
			if(ig.game.paused==0){
			
				//Sprite update
				if(this.timer.delta() > this.frameTime){
					this.frame++;
					if(this.frame > this.maxFrame){ this.frame=this.minFrame; };	
					this.timer.set(0);
				}
							
				//Spidio
				if(this.type==3){

					if(this.delayTimer.delta() > 5 && ig.game.timer.delta() < 0 && ig.game.finished == 0 ){
						this.speed = 16;
						this.delayTimer.set(0);
						this.delayTimer.pause();
					}
				
					//Movement
					this.x = this.x + ig.global.game.speed * this.speed * this.type;		
					
					//Out of screen (right)
					if(this.x >= ig.global.game.width && this.finished==0){ 
					
						ig.game.changeScore(-180,0.90 * ig.global.game.width,this.y + this.h[this.frame]/2);

						this.x = - 2 * this.w[this.frame];
						this.y = (0.15 + 0.5 * Math.random()) * ig.global.game.height;
						this.speed = 0;
						ig.game.getEntityByName('bikers').reorder();
						this.delayTimer.unpause();	
						
					};
					
					//Out of screen (left, when is coming back)
					if(this.x <= - 2 * this.w[this.frame] && this.finished==0){ 

						this.y = (0.15 + 0.5 * Math.random()) * ig.global.game.height;
						this.speed = 0;
						ig.game.getEntityByName('bikers').reorder();
						this.delayTimer.unpause();	
						
					};					
					
					//Mouse Event
					if(
						click.x > this.x && click.x < this.x + this.w[this.frame] && 
						click.y > this.y && click.y < this.y + this.h[this.frame] &&
						this.speed > 0 &&
						ig.game.finished == 0
					){
						var hit = ig.game.getEntityByName('bikers').HitTest(click.x,click.y);		
						if(hit == this.id){
							var extraPoints = 50 + Math.round(180 * this.x/ig.global.game.width);
							ig.game.changeScore(extraPoints,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);

							var newSpeed = - 2 * this.speed;
							this.speed = 0;
							TweenMax.to(this, 0.5, {speed : newSpeed, ease:Linear.easeNone});									
							
						};			
						clearMouse();							
					};		
				}		

				//Normal riders including Movistar riders
				if(this.type != 3){
								
					//Movement
					this.x = this.x + this.speed * ig.global.game.speed;
					
					//Bikers cannot be over the max distance which valor is between 0.15 and 0.2 randomly.
					if(this.x >= ig.global.game.width * (0.15 + Math.random()*0.05) && ig.game.finished == 0 && this.finished==0 && this.sprint==0){ 
						this.speed = - (4 + Math.random() * 4) 
					};

					//Bikers cannot be under the min distance which valor is between 0 and -2 randomly.
					if(this.x < -ig.global.game.width * (Math.random()*0.2) && ig.game.finished == 0 && this.finished==0 && this.sprint==0){
						this.speed = 4 + Math.random() * 4 
					};
					
					//Random stop moments
					if(this.storedSpeed == 0 && this.speed != 0 && this.sprint == 0){
						var stop = Math.round(Math.random() * 100);
						if(stop == 100){
							this.storedSpeed = this.speed;
							this.speed = 0;	
							this.delayTimer.set(Math.random() * 2);
							this.delayTimer.unpause();						
						}
					}
					//Reanude moving			
					if(this.delayTimer.delta() > 0 && this.storedSpeed != 0 && this.sprint == 0){
						this.speed = this.storedSpeed;
						this.storedSpeed = 0;
						this.delayTimer.set(0);						
						this.delayTimer.pause();						
					}
							
				}		

				//Only non movistar bikers
				if(this.type < 3){

					//Non movistar bikers final Sprint
					if(ig.game.timer.delta() > ig.global.game.finalSprint && this.sprint==0){ 
						this.sprint = 1;
						this.speed = 12;
					};

					//Out of screen after sprint
					if(this.x >= ig.global.game.width && this.finished==0 && this.sprint==1){ 
						ig.game.changeScore(-90,0.90 * ig.global.game.width,this.y + this.h[this.frame]/2);
						this.x = - this.w[this.frame];
						this.speed = 4 + Math.random() * 4;
						this.sprint = 0;
					};	
				
					//Mouse Events
					if(
						click.x > this.x && click.x < this.x + this.w[this.frame] && 
						click.y > this.y && click.y < this.y + this.h[this.frame] &&
						ig.game.finished == 0 &&
						this.finished == 0
					){
						if(this.x < ig.global.game.width * 0.20){ //Mouse Click penalize when the biker is in bikers' bunch (peloton).
							var hit = ig.game.getEntityByName('bikers').HitTest(click.x,click.y);		
							if(hit == this.id){
								if(this.sprint == 0){
									ig.game.changeScore(-30,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);
								}
								clearMouse();	
							};
						};
						if(this.x >= ig.global.game.width * 0.20 && this.speed>=0){ //Mouse Click give potins when the biker is sprinting
							var hit = ig.game.getEntityByName('bikers').HitTest(click.x,click.y);		
							if(hit == this.id){
								if(this.x > ig.global.game.width * 0.45){
									this.speed = 0;
									var newX = this.x - 0.2 * ig.global.game.width;
									TweenMax.to(this, 0.5, {x: newX, ease:Linear.easeNone});
								}else{
									this.speed = 16;
									this.sprint = 0;
									TweenMax.to(this, 0.5, {x: 0, onComplete: function(){this.speed = 4 + Math.random() * 4; }, ease:Linear.easeNone});							
									ig.game.changeScore(60,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);
								};
								clearMouse();	
							};			
						};							
					};
								
				};
						
				//Movistar bikers						
				if(this.type==4){ 
												
					//Movistar bikers Sprint
					if(ig.game.timer.delta() > ig.global.game.finalSprint && this.sprint == 0){   /* Actives sprint where finalSprint is reached */
						this.sprint = 1;
						this.speed = 0;
						TweenMax.to(this, 1, {x : 0.30 * ig.global.game.width, ease:Linear.easeNone});	
						//Hand animation
						this.hand = new ig.UIObject("Hand.png", ui, this.x + 0.8 * this.w[this.frame], this.y + 0.7 * this.h[this.frame], 0.2 * this.w[this.frame],1);
						var newY = 1.04 * this.hand.y;
						TweenMax.to(this.hand, 0.8, {alpha : 0.5, y : newY, repeat:-1, yoyo:true, ease:Linear.easeNone});						
					};

					//Change hand x if hand object exists
					if(this.hand){
						this.hand.x = this.x + 0.8 * this.w[this.frame];
					}				

					//Out of screen after sprint (In final sprint this won't work)
					if(this.x >= ig.global.game.width && ig.game.timer.delta() < ig.global.game.finalSprint && this.finished==0 && this.sprint==1){ 
						ig.game.changeScore(200,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);
						this.x = - this.w[this.frame];
						this.speed = 4 + Math.random() * 4;
						this.sprint = 0;
						this.hand = null;
					};						
					
					//Mouse Events
					if(
						click.x > this.x && click.x < this.x + this.w[this.frame] && 
						click.y > this.y && click.y < this.y + this.h[this.frame] &&
						ig.game.finished == 0 &&
						this.finished == 0
					){
						if(this.x >= ig.global.game.width * 0.20 && this.sprint==1){ //Mouse Click give points when the biker is sprinting
							var hit = ig.game.getEntityByName('bikers').HitTest(click.x,click.y);	
							if(hit == this.id){
								this.speed = 0;
								if(this.x < ig.global.game.width * 0.9){ 								
									var newX = this.x + 0.05 * ig.global.game.width;
								}else{
									var newX = this.x + 0.15 * ig.global.game.width;
								}
								if(newX < ig.global.game.width){ 
									TweenMax.to(this, 0.2, {x: newX, ease:Linear.easeNone}); 
								}else{ /* If newX is bigger than ig.global.game.width we dont use newX because it causes a bug */
									TweenMax.to(this, 0.2, {x: ig.global.game.width, ease:Linear.easeNone}); 
								}	
								clearMouse();	
							};			
						};		
					};						
					
				}			

				//When bikers cross finishLine
				if(
					ig.game.timer.delta() > 0.1 &&
					this.finished == 0 &&
					this.x + this.w[this.frame] > ig.game.getEntityByName('background').finishLine1.minX[0] + ig.game.getEntityByName('background').finishLine1.w
				){
					this.finished = 1;
					this.speed = Math.abs(this.speed);
					if(this.speed = 0){ this.speed = 4 + Math.random() * 4; };
					if(this.type==4){
						switch(ig.game.getEntityByName('bikers').ranking.length){
							case 0 : ig.game.changeScore(1200,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);break;
							case 1 : ig.game.changeScore(800,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);break;
							case 2 : ig.game.changeScore(400,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);break;
						}
					}
					ig.game.getEntityByName('bikers').ranking.push(this.type);
				}
				
				
				//Accelerate bikers when race is finished
				if(this.finished == 1){
					this.speed = 40;
				}
				
			};	
			
				
		}
			
	});
	

});