/*
* ImpactJS Obstacles Animation Plugin
* Alex Maestre - Novembre 2014
*
*/
ig.module(
    'plugins.obstacleAnimation'
)
.requires(
	'impact.image',
	'impact.timer'
)

.defines(function() { "use strict";
	
	ig.ObstacleAnimation = ig.Class.extend({
	
		atlasPNG : new ig.Image( 'media/img/obstacles.png' ),		
		
		delayTimer : null, cadenceTimer : null, timer: null, finished : 0, sprint : 0, storedSpeed : 0,
		type : null, minFrame : 1, maxFrame : 1, frame : 1, frameTime : 1, id: null, speed : null, cadence: 0,
		speedY : null, sx : [], sy : [], sw : [], sh : [], w : [], h : [], x:[], y : [], 
		
		init : function(id, type, minFrame, maxFrame, frameTime, x, y, finalHeight){
			var filename = '';
			switch(type){
				case 5 : 
					filename = 'boy'; 
					this.cadenceTimer = new ig.Timer();
					this.cadence = 16 - 6 *(ig.game.level/50);
					this.cadenceTimer.set(this.cadence);	
					this.speed = -(15 + Math.random() * 5); 
					this.speedY = -(10 + Math.random() * 5); 
					break;
				case 6 : 
					filename = 'motorbike'; 
					this.delayTimer = new ig.Timer();
					this.delayTimer.set(0);	
					this.cadence = 20 - 10 *(ig.game.level/50);
					this.cadenceTimer = new ig.Timer();					
					this.cadenceTimer.set(this.cadence);					
					this.speed =  -(4 + Math.random() * 4); 
					break;
			}
			this.id = id;
			this.type = type;
			this.timer = new ig.Timer();
			this.frameTime = frameTime;
			this.minFrame = minFrame;
			this.maxFrame = maxFrame;
			this.x = x;
			this.y = y;			
			for(var f=minFrame; f<=maxFrame; f++){
				for(var i=0; i<=obstacles.length-1; i++){
					if(obstacles[i].filename == filename + f + '.png'){
						this.sx[f] = obstacles[i].frame.x;
						this.sy[f] = obstacles[i].frame.y;
						this.sw[f] = obstacles[i].frame.w;
						this.sh[f] = obstacles[i].frame.h;
						this.h[f] = Math.round(finalHeight);
						this.w[f] = Math.round(finalHeight * obstacles[i].frame.w/obstacles[i].frame.h);	
					}
				}				
			}
		},
		
		draw : function (){
			var ctx = ig.system.context;
			ctx.drawImage(this.atlasPNG.data, this.sx[this.frame], this.sy[this.frame], this.sw[this.frame], this.sh[this.frame], this.x, this.y, this.w[this.frame], this.h[this.frame]);	
		},
			
		update : function(){
			
			if(ig.game.paused==0){
			
				//Sprite update
				if(this.timer.delta() > this.frameTime){
					this.frame++;
					if(this.frame > this.maxFrame){ this.frame=this.minFrame; };	
					this.timer.set(0);
				}

				//Motorbike
				if(this.type == 6 && ig.game.level>=3){
			
					//Movement	
					if(this.cadenceTimer.delta() > 0){	
					
						this.x = this.x + this.speed * ig.global.game.speed;
										
						//Random stop moments
						if(this.storedSpeed == 0 && this.speed != 0){
							var stop = Math.round(Math.random() * 100);
							if(stop == 100){
								this.storedSpeed = 1;
								this.speed = 0;	
								this.delayTimer.set(Math.random() * 2);
								this.delayTimer.unpause();						
							}
							if(stop < 10){
								this.speed = 4 + Math.random() * 12;			
							}						
						}
						
						//Reanude moving			
						if(this.delayTimer.delta() > 0 && this.storedSpeed != 0){
							this.speed = 4 + Math.random() * 12;
							this.storedSpeed = 0;
							this.delayTimer.pause();						
						}
					
						//Out of screen
						if(this.x >= ig.global.game.width && this.speed>0 && ig.game.finished==0){ 
							this.x = - this.w[this.frame];
							this.y = (0.15 + 0.5 * Math.random()) * ig.global.game.height;
							this.speed = this.speed = 4 + Math.random() * 12;
							this.cadenceTimer.set(this.cadence);						
						};	
					};
									
					//Dissapear when game finishes
					if(ig.game.finished == 1){
						if(this.x > 0){
							this.speed = 40;
						}else{
							this.speed = 0;
						};
					}								
							
				}		

				//Boy
				if(this.type == 5 && ig.game.level>=6){
							
					//Movement	
					if(this.cadenceTimer.delta() > 0){	
						this.x = this.x + this.speed * ig.global.game.speed;
							if(this.y < 0.2 * ig.global.game.height){
								this.speedY = 10 + 5 * Math.random();
							}
							if(this.y > 0.7 * ig.global.game.height){
								this.speedY = -(10 + 5 * Math.random());
							};						
						
						this.y = this.y + this.speedY * ig.global.game.speed;
					};
					
					//Came back
					if(this.x < 0.5 * ig.global.game.width){
						this.speed= 15 + Math.random() * 5;
					}
					
					//Out of screen
					if(this.x > ig.global.game.width && this.speed>0 && ig.game.finished==0){
						this.y = (0.15 + 0.5 * Math.random()) * ig.global.game.height;
						this.speed = - this.speed;
						this.cadenceTimer.set(this.cadence);				
					}
					
					//Restart movement
					if(this.cadenceTimer.delta() > 0 && this.speed==0 && ig.game.finished == 0 ){
						this.speed = -(15 + Math.random() * 5);
					}					
					
					//Dissapear when game finishes
					if(ig.game.finished == 1){
						if(this.x < ig.global.game.width){
							this.speed = 40;
						}else{
							this.speed = 0;
						};
					}					
									
				};					
				
				//Common mouse Events
				if(
					click.x > this.x && click.x < this.x + this.w[this.frame] && 
					click.y > this.y && click.y < this.y + this.h[this.frame] &&
					ig.game.finished == 0 &&
					this.finished == 0
				){
					if(this.type == 5){var penalizacion = -100;}
					if(this.type == 6){var penalizacion = -200;}
					var hit = ig.game.getEntityByName('bikers').HitTest(click.x,click.y);		
					if(hit == this.id){
						ig.game.changeScore(penalizacion,this.x + this.w[this.frame]/2,this.y + this.h[this.frame]/2);
						clearMouse();	
					};			
				};					
				
			};	
			
				
		}
			
	});
	

});